
/*
 * REPEATER CONTROLLER FOR RTQ NETWORK
 *
 * VERSION 0.9  - Initial release
 * VERSION 0.91 - Bug in monitor command, Locked in monitor when activated.
 * VERSION 0.92 - Bug, patch can be opened when link are active.
 *                Reset processor command added
 *                Group open command modified 2* instead of 1*
 *                Monitor/open command modifed 156 for monitor, * after to open.
 *                *** PATCH CANNOT HAVE JUST * OR # FOR OPENING. MUST BE CODED SUCH *678 AND # OR #678.
 *                You can reoopen already opened repeater (confirm if the repeater are there)
 * VERSION 0.92a  Bug in reset command, will hang-up controller.
 * VERSION 0.95   Eliminate some command from UHF, allow only on VHF. (Patch, programming, etc...)
 * VERSION 0.96   -Support for new harware (downware compatible)
 *                -Code cleanup
 *                -New set of tech command
 * VERSION 0.99 - For A.L.A.N. implementation some command must be
 *                simplified.
 *                 -No more UHF TOT
 *                 -No more link monitor (Never used by ham)
 *
 *
 *
 * VHF tot a ameliorer
 * prend les command DTMF meme si squelch encore dessus
 *     (en cas de panne, squelch reste ouvert.)
 *
 *
 *
 *  DEFAULT COMMAND:
 * *???     Open patch
 * #???     Close patch
 * 1xy*     Open Link x=repeater y=network
 * 1xy#     Close link x=repeater y=network
 * 10y*     Open all link y=network
 * 10y#     Close all link y=network
 * 2*       Open local repeater group (Inactivity timeout disabled in this mode)
 * 2#       Close local repeater group
 * Axyy     Programming x=param number yy=hex value
 *
 * DACxy    Master command, x=repeater y=command
 *                          0=all      0 = close link
 *                                     1 = open link (permanent)
 *                                     2 = open emergency link (permanent, with beep)
 *                                     5 = Loopback tone 10 sec.
 *                                     A = Enable programmation
 *                                     B = Disable programmation
 *                                     DA = Reset Processor and all eeprom config
 *
 * Programming parameter:
 * 0  Tail length in 73.23ms tick (default 1.75sec)
 * 1  Beep delay in 73.23ms tick (default 0.7sec)
 * 2  Link inactivity timer in 18.75s tick (0=no TOT) (default 5 min)
 * 3  VHF CW IDENT in 18.75s tick (default 20 min)
 * 4  VHF TOT delay in 18.75s tick (0=no TOT) (default 5 min)
 * 5  UHF TOT delay in 18.75s tick (0=no TOT) (default 5 min)
 * 6  Length of patch dtmf code. ex: * is one, *123 is four. (default is 2)
 *
 *	Pin assignments
 *
 *	Port B
 *	0	SQ VHF (positive)
 *	1   SQ UHF
 *	2	PTT VHF (positive)
 *	3	PTT UHF
 *	4	Beep out
 *	5	DTMF toggle
 *	6	Link pin (5v when link are open)
 *	7	I/O pin (Processor information exchange)
 *
 *	Port A
 *  0   DTMF bit 0
 *  1   DTMF bit 1
 *  2   DTMF bit 2
 *  3   DTMF bit 3
 *  4   DTMF data valid
 *
 *
 */

//#define ROUYN 1
//#define LASARRE 1
#define VALDOR 1

#include <pic.h>

/*{{{  Variable*/

#define FALSE 0
#define TRUE -1

#define DTMFSIZE 16          // DTMF buffer is 6 digits

/*{{{  Parameter default value*/


// PARAMETER DEFAULT VALUE
//--------------------------
#define DTMFDELAY 6         // 6*50ms= 300ms delay when scanning DTMF
#define IDNOWTOT 11         // When IDTOT expire, CW are send 0.75 second after dropped squelch
#define RADIOALRM 1638      // Radio send bip each 2 minute(s) after radio TOT



// LINK ID NUMBER (FIXED IN CPU)
//--------------------------------
#define NETWORK_ID  6

#ifdef VALDOR
#define REPEATER_ID 4
#endif

#ifdef ROUYN
#define REPEATER_ID 5
#endif

#ifdef LASARRE
#define REPEATER_ID 6
#endif
/*}}}  */

/*{{{  EEPROM memory map*/

// DEFAULT VALUE
//-----------------
#define TAILDELAY 0x18        // 2 sec tail
#define BEEPDELAY 0x0A        // 0.5sec for beep
#define INACTLINKTOT 16       // After 5min of inactivity, the link will close
#define IDTOT 64              // After 20min of VHF inactivity, repeater send CW ID
#define VHFTOT  16            // VHF radio TOT limit is 5min (Constant Squelch)
#define NDPATCH 2             // Just *x to open patch (two digit)

// EEPROM MEMORY MAP
//--------------------
#define EE_TAIL       0         // Tail in 73.23ms
#define EE_BEEP       1         // Beep delay in 73.23ms
#define EE_LINKTOT    2         // Link inactivity timer in 18.75s tick
#define EE_IDTOT      3         // VHF CW IDENT in 18.75s tick
#define EE_VHFTOT     4         // VHF TOT delay in 18.75s tick
#define EE_FREE       5         // Free
#define EE_NDPATCH    6         // Number of digit in patch code


void EEPromWrite(addr, value) {
    while(WR)continue;
    EEADR=(addr);
    EEDATA=(value);
    GIE=0;
    WREN=1;
    EECON2=0x55;
    EECON2=0xAA;
    WR=1;
    WREN=0;
}

unsigned char EEPromRead(unsigned char addr) {
    return ((EEADR=(addr)),(RD=1),EEDATA);
}
/*}}}  */


/*{{{  Misc define*/

// MISC DEFINE
#define	PORTBIT(adr, bit)	((unsigned)(&adr)*8+(bit))
#define CLEARWATCHDOG() asm("clrwdt")
#define DIGIT_0 10
#define DIGIT_1 1
#define DIGIT_2 2
#define DIGIT_3 3
#define DIGIT_4 4
#define DIGIT_5 5
#define DIGIT_6 6
#define DIGIT_7 7
#define DIGIT_8 8
#define DIGIT_9 9
#define DIGIT_A 13
#define DIGIT_B 14
#define DIGIT_C 15
#define DIGIT_D 0
#define DIGIT_STAR  11
#define DIGIT_DIESE 12
/*}}}  */

/*{{{  Port bit definition*/

// PORT BIT DEFINITION
//----------------------
static bit	SqVhf      @	PORTBIT(PORTB, 0);
static bit	SqUhf      @	PORTBIT(PORTB, 1);
static bit	PttVhf     @	PORTBIT(PORTB, 2);
static bit	PttUhf     @	PORTBIT(PORTB, 3);
static bit	Beep       @	PORTBIT(PORTB, 4);
static bit	DTMFtog    @	PORTBIT(PORTB, 5);
static bit	UhfGate    @	PORTBIT(PORTB, 6);
static bit	Led        @	PORTBIT(PORTB, 7);

static bit	DtmfValid  @	PORTBIT(PORTA, 4);
/*}}}  */


// DTMF VARIABLE
//----------------
unsigned char DTMF[DTMFSIZE];   // DTMF buffer
unsigned char DTMFptr;          // DTMF pointer in buffer
volatile char DTMFdelay;        // Delay on 50ms tick before change audio source
         bit  DTMFlock;         // TRUE is DTMF scanning locked on audio source


// TAIL/BEEP/CW VARIABLE
//------------------------
volatile char BeepDelay;        // Beep delay in 50ms tick
volatile char TailDelay;        // Tail delay in 50ms tick
          bit IdNow;            // Send CW ID NOW!!! (1 second after squelch drop)
         char Freq;             // Beep parameter (Freq & Dura)
         char Dura;
         char CWSpeed;          // Tone(beep) duration for CW id


// TOT TIMER
//------------
volatile unsigned int InactLinkTOT;     // Inactive link TOT
volatile unsigned int IdTOT;            // Repeater ID TOT
volatile unsigned int VhfTOT;           // Radio TOT
volatile unsigned int timer;            // General timer

// REPEATER VARIABLE
//--------------------
bit LinkOpen;                           // Link are open
bit LinkCloseing;                       // Set if link must be close after CW ID
bit PatchEnable;                        // TRUE if patch enabled
bit LinkTOTenable;                      // Set if Link TOT is enable
bit ProgramEnable;                      // Enable controller programming
bit AcknoledgeBeep;                     // Send acknoledge roger beep for next


// MISC
//-------
char temp1,temp2,temp3,temp4;
char const conv[16] = { 13,1,2,3,4,5,6,7,8,9,0,14,15,10,11,12 };

/*}}}  */

/*{{{  Timer interrupt*/

void interrupt TimerInterrupt() {
    if(DTMFdelay) DTMFdelay--;          // Process DTMF scan delay
    if(BeepDelay) BeepDelay--;          // Process Beep delay
    if(TailDelay) TailDelay--;          // Process Tail delay
    if(InactLinkTOT) InactLinkTOT--;    // Link inactivity time-out timer
    if(IdTOT) IdTOT--;                  // Repeater ID time-out timer
    if(VhfTOT) VhfTOT--;                // VHF radio time-out timer
    timer++;

    T0IF = 0;
}
/*}}}  */


/*{{{  BEEP Output roger beep*/

extern void Sound();

/*{{{  Sound() - Output sound on RB.4*/
#asm

;-------------------------------------------------------------------------
;--- Sound and Silence SubRoutine
; Entry		: _Freq,_temp2
; Uses 		: _temp1,_temp2,_temp3,_temp4
;-------------------------------------------------------------------------

spkr    equ     6       ; PORTB
beep_m  equ     16      ; (PORTB.4)

_Sound
    movlw beep_m        ; Load Beep Mask
    movwf _temp3
	movf _Freq,w        ; Copy Frequency Data
	movwf _temp2
	btfsc 3,2	        ; Skip if Frequency is Not Zero
	clrf _temp3		    ; Make Silence
	clrf _temp1		    ; Reset Low Counter
	movf _temp3,w       ; Determine Silence or Beep
	movwf _temp4

sndmk_
    movf _temp4,w       ; Make Sound
    xorwf spkr,f
	clrwdt		        ; Clear Watchdog (Replace nop)
	nop			        ; For Timming Purpose
	decfsz _temp2,f       ; Loop Before Toggle of Beep Pin
	goto stpsnd_

	movf _Freq,w        ; Reload Frequency of Beep (Set High Counter)
	movwf _temp2
	movf _temp3,w       ; Determine Silence or Beep
	movwf _temp4

sndclk_
	movlw 1             ; Sub 1 from _temp1 (to use Carry flag)
	subwf _temp1,f
	btfsc 3,0           ; Jmp if Borrow
	goto  sndtim_
	movlw 1             ; Sub 1 from _Dura (to use Carry flag)
	subwf _Dura,f
    btfss 3,0	        ; Skip if Borrow
	return			    ; Sub-Routine Complete
	goto  sndmk_	    ; Else Continue...

stpsnd_
    clrf _temp4		    ; Make Silence
	goto sndclk_	    ; Check Duration

sndtim_
    clrwdt		        ; Clear Watchdog (Replace nop)
	nop			        ; For Timming Purpose
	goto sndmk_		    ; Continue...

#endasm
/*}}}  */


void Tone() { Dura=15; Sound(); Freq=0; Dura=3; Sound(); }

void RogerBeep() {
    if(LinkOpen  || AcknoledgeBeep) {  // Lien ouvert: deux tone pareil
        Freq=40; Tone();
        Freq=40; Tone();
        if(AcknoledgeBeep) { Freq=40; Tone(); AcknoledgeBeep=0; }
    } else {
        Freq=40; Tone();               // Rogerbeep normal: deux tone montant
        Freq=33; Tone();
        if(SqUhf) {                 // Activite UHF ou beep de validation: un tone supplementaire
            Freq=25; Tone();
        }
    }
}
/*}}}  */

/*{{{  CWID CW identification*/

/*{{{  Dit, Dah and space*/

#ifdef VALDOR
void Dit() { Freq=30; Dura=CWSpeed; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
void Dah() { Freq=30; Dura=CWSpeed*3; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
#endif
#ifdef ROUYN
void Dit() { Freq=40; Dura=CWSpeed; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
void Dah() { Freq=40; Dura=CWSpeed*3; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
#endif
#ifdef LASARRE
void Dit() { Freq=34; Dura=CWSpeed; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
void Dah() { Freq=34; Dura=CWSpeed*3; Sound(); Freq=0; Dura=CWSpeed; Sound(); }
#endif
void Space() { Freq=0; Dura=CWSpeed*3; Sound(); }
/*}}}  */


//-------------------------------------------
// CW IDENTIFICATION
//-------------------------------------------
void CWID(char speed) {
    CWSpeed = speed;
    Space();
    Dit();
    Dit();
    Dit();
    Dah();
    Space();
    Dit();
    Space();
    Dit();
    Dit();
    Dah();
    Dah();
    Dah();
    Space();
    Dit();
    Dah();
    Dit();
    Space();

#ifdef VALDOR
    Dah();
    Dit();
    Dah();
    Dah();
    Space();
    Dit();
    Dah();
    Dit();
    Dit();
#endif
#ifdef ROUYN
    Dah();
    Dah();
    Dah();
    Space();
    Dah();
    Dit();
#endif
#ifdef LASARRE
    Dit();
    Dit();
    Dit();
    Space();
    Dit();
    Dah();
    Dit();
    Dit();
#endif
    Space();
}
/*}}}  */

/*{{{  LINK Open/Close*/

/*{{{  OpenLink*/

void OpenLink() {
    if(PatchEnable) return;
    if(LinkOpen) return;

    InactLinkTOT = 256*EEPromRead(EE_LINKTOT); // Normal time-out

    LinkOpen     = TRUE;                    // Link are open
    LinkCloseing = FALSE;

    IdNow = TRUE;                           // Send CWID now!
    IdTOT = IDNOWTOT;
}
/*}}}  */

/*{{{  CloseLink*/

void CloseLink() {
    if(LinkOpen==0) return;
    LinkCloseing = TRUE;

    IdNow = TRUE;                                // ID sending is imminent
    IdTOT = IDNOWTOT;
}
/*}}}  */

/*}}}  */

/*{{{  DTMF handling*/

/*{{{  ProcessDTMF - Identify dtmf command*/

//--------------------------------------------------
// IDENTIFY DTMF COMMAND AND PROCESS IT
//
//
// DEFAULT COMMAND:
// *        Open patch
// #        Close patch
// 1xy*     Open Link x=repeater y=network
// 1xy#     Close link x=repeater y=network
// 10y*     Open all link y=network
// 10y#     Close all link y=network
// 1xy      Monitor link for 10sec * to open when monitor
// 2*       Open local repeater group (Inactivity timeout disabled in this mode)
// 2#       Close local repeater group
// Axyy     Programming x=param number yy=hex value
// DACxy    Master command, x=repeater y=command
//                          0=all      0 = close link
//                                     1 = open link (permanent)
//                                     2 = open emergency link (permanent, with beep)
//                                     5 = Loopback tone
//                                     A = Enable programmation
//                                     B = Disable programmation
//                                     DA = Reset Processor and reset eeprom to default
//
// Programming parameter:
// 0  Tail length in 73.23ms tick (default 1.75sec)
// 1  Beep delay in 73.23ms tick (default 0.7sec)
// 2  Link inactivity timer in 18.75s tick (0=no TOT) (default 5 min)
// 3  VHF CW IDENT in 18.75s tick (default 20 min)
// 4  VHF TOT delay in 18.75s tick (0=no TOT) (default 5 min)
// 5
// 6  Length of patch dtmf code. ex: * is one, *123 is four. (default is 2)
//
// DTMF CHART:
//  0123456789ABCDEF
// "D1234567890*#ABC"
//--------------------------------------------------

void ProcessDTMF() {
    unsigned char i;

    // COMMAND FOR VHF ONLY
    //-----------------------
    switch(DTMF[0]) {
        /*{{{  *      : Open patch or link*/
//
// OPEN LINK OR PATCH
//
// IF LINK ARE MONITORED: * ONLY, WILL OPEN IT WITH CW ID.
// OTHERWISE * WITH ANY CODE AFTER WILL DISABLE TAIL FOR PATCH.
//
//

case DIGIT_STAR:    if( DTMFptr >= EEPromRead(EE_NDPATCH) ) {
                        LinkOpen=FALSE;
                        PatchEnable=TRUE;
                    }
                    break;
/*}}}  */

        /*{{{  #      : Close patch*/

//
// PATCH CLOSE (#xxx)
//
case DIGIT_DIESE:   PatchEnable=FALSE;          // Patch OFF (#)
                    break;
/*}}}  */

        /*{{{  Axyy   : Controller parameter programming*/

//
// PROGRAMING THE CONTROLLER
// Dxyy - x=EEPROM ADDRESS yy=HEX VALUE
//
case DIGIT_A:   if(ProgramEnable) {
                    EEPromWrite(conv[DTMF[1]], (conv[DTMF[2]]*16)+conv[DTMF[3]]);      // Parameter set
                    GIE=1;
                    AcknoledgeBeep = 1;
                }
                break;
/*}}}  */

        /*{{{  2x     : Regional group open/close*/

//
// REGIONAL OPEN
// THIS CODE WILL OPEN ALL REPEATER WITH THIS CODE, NO LINK TIME-OUT.
//
case DIGIT_2:   if(DTMF[1]==DIGIT_STAR) OpenLink();                                 // Local group open/close
                if(DTMF[1]==DIGIT_DIESE) CloseLink();
                LinkTOTenable = 0;
                AcknoledgeBeep = 1;
                break;
/*}}}  */

        /*{{{  1xyz   : Open/Close link*/

//
// OPEN LINK WITH 156*, CLOSE WITH 156#, MONITOR WITH 156.
// LINK ARE ALSO OPENED WITH 106* AND 106# (ALL NETWORK #6 OPENING)
//
case DIGIT_1:       if(DTMF[2]==NETWORK_ID) {
                        if(DTMF[1]==DIGIT_0 || DTMF[1]==REPEATER_ID) {
                            if(DTMF[3]==DIGIT_STAR) OpenLink();
                            if(DTMF[3]==DIGIT_DIESE) CloseLink();
                            LinkTOTenable = 1;
                        }
                    }
                    break;
/*}}}  */

        /*{{{  DACxy  : Master programming code*/

    case DIGIT_D:   if(DTMF[1]==DIGIT_A && DTMF[2]==DIGIT_C) {
                        if(DTMF[3]==DIGIT_0 || DTMF[3]==REPEATER_ID) {
                            switch(DTMF[4]) {
                                case DIGIT_0:   CloseLink(); AcknoledgeBeep = 1; break;
                                case DIGIT_1:   OpenLink(); LinkTOTenable = 0; AcknoledgeBeep = 1; break;
                                case DIGIT_2:   OpenLink(); LinkTOTenable = 0; AcknoledgeBeep = 1; break;
                                case DIGIT_5:   /*{{{  Loopback tone*/

    for(i=0; i<7; i++) { Freq=34; Dura=255; Sound(); }
/*}}}  */
 break;
                                case DIGIT_A:   ProgramEnable = 1; AcknoledgeBeep = 1; break;
                                case DIGIT_B:   ProgramEnable = 0; AcknoledgeBeep = 1; break;
                                case DIGIT_D:   /*{{{  DACxDy - Sub command set*/

    switch(DTMF[5]) {

        // RESET EEPROM AND PROCESSOR
        case DIGIT_A:   EEPromWrite(EE_TAIL, TAILDELAY);
                        EEPromWrite(EE_BEEP, BEEPDELAY);
                        EEPromWrite(EE_LINKTOT, INACTLINKTOT);
                        EEPromWrite(EE_IDTOT, IDTOT);
                        EEPromWrite(EE_VHFTOT, VHFTOT);
                        EEPromWrite(EE_NDPATCH, NDPATCH);
                        while(1);   // Watchdog reset CPU
                        break;
    }
/*}}}  */
 break;
                            }
                        }
                    }


/*}}}  */

    }

    DTMFptr = 0;
    GIE = 1;
}
/*}}}  */

/*{{{  GetDTMF - Get string of DTMF*/

void GetDTMF() {

// VERIFYING IF DTMF MUST BE DECODED
//-----------------------------------
    if(DTMFptr!=0) {
        if( (SqVhf==0 && DTMFtog==0) || (SqUhf==0 && DTMFtog==1) ) {
            ProcessDTMF();
            return;
        }
    }

// SELECT AUDIO SOURCE: A, B OR SCAN
//-----------------------------------
    if(SqVhf==1 && SqUhf==0) { DTMFtog=0; DTMFlock=TRUE; } else
        if(SqVhf==0 && SqUhf==1) { DTMFtog=1; DTMFlock=TRUE; } else
            if(DTMFptr==0) DTMFlock=FALSE;


// TOGGLE AUDIO SOURCE IF NOT LOCKED AND DELAY EXPIRE
//-----------------------------------------------------
    if(DTMFlock==FALSE && DTMFdelay==0) {
        DTMFdelay = DTMFDELAY;                  // Reset delay
        if(DTMFtog) DTMFtog=0; else DTMFtog=1;  // Toggle audio source
    }


// STORE DTMF IF ANY, AND LOCK AUDIO SOURCE
//-------------------------------------------
    if(DtmfValid) {
        DTMFlock = TRUE;
        if(DTMFptr==DTMFSIZE) return;
        DTMF[DTMFptr++] = PORTA&15;
        while(DtmfValid) CLEARWATCHDOG();
    }
}
/*}}}  */

/*}}}  */

/*{{{  Led function*/

void LedBlinkSlow() {
    if(timer&8) Led=0; else Led=1;
}

void LedBlinkFast() {
    if(timer&2) Led=0; else Led=1;
}

/*}}}  */


/*{{{  Main*/

void main() {
    OPTION = 0x07;                      // Pull-up, timer prescale 1:256 (286.03uS at 3.58Mhz)
    INTCON = 0xA0;                      // Enable Timer interrupt
    TMR0  = 81;                         // Set timer for 50ms tick
	TRISA = 0xFF;	                    // DTMF input port
	TRISB = 0x03;	                    // SQs is input, other is output
	PORTB = 0x80;                       // Idle port

    VhfTOT      = 256*EEPromRead(EE_VHFTOT);  // Reset VHF TOT
	IdNow       = TRUE;                        // Send repeater ID NOW!
    LinkOpen    = 0;
    PatchEnable = 0;
    ProgramEnable  = 0;
    AcknoledgeBeep = 0;
    DTMFptr  = 0;
    DTMFlock = 0;

	while(1) {
        GetDTMF();
        /*{{{  SQ/PTT process with tail and rogerbeep*/


// IF VHF SQUELCH OR UHF SQUELCH WITH LINK OPEN OR MONITOR:
//-----------------------------------------------------------
if(LinkOpen&&SqUhf) UhfGate=1; else UhfGate=0;      // Open UHF gate if link open and squelch detected

if(SqVhf || (SqUhf&&LinkOpen)) {

    if(LinkOpen) {
        InactLinkTOT = 256*EEPromRead(EE_LINKTOT);  // Squelch detected, reset link TOT
        if(SqUhf==0) PttUhf=1; else PttUhf=0;       // PTT UHF if no signal come from here
    }
    if(IdNow) IdTOT = IDNOWTOT;                     // Reset ID timer, if ID is urgent.
    TailDelay=EEPromRead(EE_TAIL);                  // Reset VHF tail/beep delay
    BeepDelay=EEPromRead(EE_BEEP);
    PttVhf=TRUE;                                    // PTT VHF

} else {

// NO VHF SQUELCH, CLEAR UHF PTT. TAIL AND ROGER BEEP (NO TAIL ON PATCH)
//------------------------------------------------------------------------
    PttUhf = FALSE;                                            // Close UHF ptt
    if(PttVhf) {
        if(BeepDelay==0) { RogerBeep(); BeepDelay=255; }       // Output roger beep
        if(TailDelay==0 || PatchEnable) { PttVhf=FALSE; }      // Close VHF ptt now(patch) or delayed(normal)
    }

}
/*}}}  */

        /*{{{  Link inactivity TOT process*/

//--------------------------------------------------
// Close link after 5min of inactivity.
//--------------------------------------------------
    if(LinkTOTenable && InactLinkTOT==0 && EEPromRead(EE_LINKTOT)!=0)
        if(LinkOpen && LinkCloseing==0) CloseLink();
/*}}}  */

        /*{{{  CW ID, repeater ID or link open/close*/

//------------------------------------------------------
// Send repeater ID periodically.
//
// When 20min time-out expire, IdNow is set and ID
// can be sended after 1 second of inactivity.
//------------------------------------------------------
    if(IdTOT==0) {
        if(LinkOpen) {

            // SEND CW ID ON BOTH RADIO FOR OPENING/CLOSING LINK
            //-----------------------------------------------------
            if(IdNow) {
                IdNow = FALSE;          // Disable ID

                PttVhf=PttUhf=1;
                CWID(12);               // Send slow ID (do not send when monitor)
                if(LinkCloseing==1) {
                    /*{{{  When link close, send fast CWID and close audio gate*/

    CWID(7);                           // Send fast ID when close link (do not send when monitor)
    LinkOpen    = FALSE;               // Close audio gate
    IdTOT = 256*EEPromRead(EE_IDTOT);  // Reset normal VHF repeater ID timer
/*}}}  */

                }
                PttVhf=PttUhf=0;
            }

        } else {

            // SEND VHF NORMAL ID EACH 20 MINUTES, AFTER 0.75 SECOND WITHOUT SQUELCH
            //---------------------------------------------------------------------
            if(IdNow) {
                /*{{{  Send ID now*/

    PttVhf=1;
    CWID(7);                            // CW ID
    IdTOT = 256*EEPromRead(EE_IDTOT);  // Reset normal VHF repeater ID timer
    IdNow = FALSE;                      // No urgent ID needed
    PttVhf=0;
/*}}}  */

            } else {
                /*{{{  20min time-out expired, wait 1 second before CWID*/

    IdNow = TRUE;           // ID sending is imminent (1 second wait)
    IdTOT = IDNOWTOT;
    /*}}}  */

            }

        }
    }
/*}}}  */

        /*{{{  VHF TOT*/

//------------------------------------------------------
// When VHF SQ are keyed for more than 10 minutes, the
// repeater close link and shutdown ptt. each 2 minute
// the repeater send his CWID with TOT in CW.
//------------------------------------------------------
    if(SqVhf) {
        if(VhfTOT==0 && EEPromRead(EE_VHFTOT)!=0) {
            PttVhf=0;
            while(SqVhf) {
                if(VhfTOT==0) {
                    VhfTOT = RADIOALRM;
                    PttVhf=1;
                    /*{{{  Send repeater ID and TOT in CW*/

    CWID(8);
    Space();
    Dah(); Space();
    Dah(); Dah(); Dah(); Space();
    Dah();
/*}}}  */

                    PttVhf=0;
                }
                CLEARWATCHDOG();
            }
            VhfTOT = 256*EEPromRead(EE_VHFTOT);
        }
    } else
        VhfTOT = 256*EEPromRead(EE_VHFTOT);
/*}}}  */

        CLEARWATCHDOG();
        if(SqVhf||SqUhf) LedBlinkFast(); else LedBlinkSlow();
	}
}
/*}}}  */

